#!/usr/bin/env python
#-*- coding:utf-8 -*-


# Step 1: Vectorise Raster Material ##########################################


### options and flags

#%option
#% key: reference_raster
#% type: string
#% gisprompt: old,cell,raster
#% description: High resolution reference raster map (must be one)
#% required : yes
#% multiple: no
#%end

#%option
#% key: reference_coi_rasters
#% type: string
#% gisprompt: old,cell,raster
#% description: High resolution dichotomic reference map(s) (can be mutliples)
#% required : yes
#% multiple: yes
#%end

#%option
#% key: classification_rasters
#% type: string
#% gisprompt: old,cell,raster
#% key_desc:
#% description: Low resolution dichotomic classification maps to be assessed d.(should be multiples)
#% required : yes
#%end


# import modules
# import sys
import grass.script as grass
import pprint

# variables
# output: vectors of reference raster, reference coi rasters and classification
vectorised_maps = []










# Step 1: Vectorise Raster Material ##########################################
def VectoriseReferences():
    """
    Preparative action (a) for the creation of low resolution vector grids:
    converting high-resolution reference and reference-CoI raster maps
    separately to points (=centroid of the cell)
    """


# Variables from options #####################################################
### input: reference raster map, reference CoI raster map

        # This name is very important. Make it smart(-er)?
    # reference_raster: there should be only one
    reference_raster = options['reference_raster']

        # This name is very important. Make it smart(-er)?
    # reference_coi_rasters: there can be multiples
    reference_coi_rasters = options['reference_coi_rasters'].split(',')

### output: reference vector points, reference CoI vector points
#############################################################################


    print "\n > Converting high resolution reference pixels to points (centroids)"


    # loop over reference Class of Interest maps
    for RCoIR in reference_coi_rasters:


        # match region to reference Class of Interest
        grass.run_command('g.region',\
        flags = 'a',\
        quiet = True,\
        rast = RCoIR)
        print """\n   * Region matched to '%s'"""\
         % RCoIR


        # convert all high resolution reference pixels to vector points
        # (actually to centroids) for each coi map separately


##############################################################################
#

# e.g., add the "pareto_" prefix.

# check:    if not RCOiR.find("pareto"):
#                reference_points = "pareto_" + RCoIR

# similar check for "coi -> points"

#
##############################################################################


        ### This name is very important. Currently static. Make it smart(-er)?
        reference_points = "pareto_ref_points___" + RCoIR

        grass.run_command('r.to.vect',\
        flags = '-o',\
        quiet = True,\
        input = reference_raster,\
        feature = "point",\
        output = reference_points)
        print """   * All reference pixels for '%s' converted to points"""\
         % RCoIR

        #
        vectorised_maps.append(reference_points)


        # convert high resolution reference CoI pixels to vector-points
        # (actually to centroids)

        ### This name is very important. Currently static. Make it smart(-er)?
        reference_coi_points = "pareto_ref_coi_points___" + RCoIR

        grass.run_command('r.to.vect',\
        flags = '-o',\
        quiet = True,\
        input = RCoIR,\
        feature = "point",\
        output = reference_coi_points,\
        )
        print """   * Reference Class of Interest pixels for %s converted to points"""\
         % RCoIR


        #
        vectorised_maps.append(reference_coi_points)


        # drop column "value" from vectorised maps

        ######################################################################
        #        # use "v.db.dropcolumn" for grass70 !

        #        grass.run_command('v.db.dropcolumn',\
        #        quiet = True,\
        #        map = reference_points,\
        #        column = "value")

        #        grass.run_command('v.db.dropcolumn',\
        #        quiet = True,\
        #        map = reference_coi_points,\
        #        column = "value")
        ######################################################################

        # drop "value" column from reference points, reference_coi_points
        grass.run_command('v.db.dropcol',\
        quiet = True,\
        map = reference_points,\
        column = "value")

        grass.run_command('v.db.dropcol',\
        quiet = True,\
        map = reference_coi_points,\
        column = "value")

        print """   * Column "value" not required; dropped for '%s' and '%s'"""\
         % (reference_points, reference_coi_points)


        ######################################################################
        #### """ add *later* as many "gridcell" columns as
        #### the number of created low resolutin vector grids """
        ######################################################################


def VectoriseClassifications():
    """Preparative action (b) for the creation of low resolution vector grids:
    vectorise low resolution classification raster map(s)
    """


# Variables from options #####################################################

### input: classification raster map(s)

    # get classification rasters from options
    classification_rasters = options['classification_rasters'].split(',')

    # if not given then get from g.mlist
    if classification_rasters == '':
        classification_rasters_raw = grass.read_command('g.mlist',\
                                                type = "rast",\
                                                pat = "pareto_classification*")
        classification_rasters = classification_rasters_raw.splitlines()
##############################################################################


    print "\n > Vectorising classified dichotomic maps"


    # add a check here whether to vectorise or not, like (pseudocode follows):
    #   for CR in classification_rasters:
    #       if CR is not vector:
    #           vectorise # like below
    #       else:
    #           skip vectorisation


    for cR in classification_rasters:


        # use a name convention... ?
        cV = "pareto_classification_V___" + cR


        # match region to reference Class of Interest!?
            # really required for "r.to.vect feature=area" ? #
        grass.run_command('g.region',\
        flags = 'a',\
        quiet = True,\
        rast = cR)
        print """\n   * Region matched to '%s'"""\
         % cR


        # vectorise low resolution classification raster
        grass.run_command('r.to.vect',\
        flags = '-o',\
        quiet = True,\
        input = cR,\
        feature = "area",\
        output = cV)
        print """   * Classification raster '%s' vectorised"""\
         % cR


        #
        vectorised_maps.append(cV)


        # drop unnecessary column "value" from vectorised maps

    #    # note: use "v.db.dropcolumn" for grass70
    #    grass.run_command('v.db.dropcolumn',\
    #    quiet = True,\
    #    map = CRV,\
    #    column = "value")

        # drop "value" column from vectorised classification map(s)
        grass.run_command('v.db.dropcol',\
        quiet = True,\
        map = cV,\
        column = "value")


        print """   * Column named "value" not required; dropped from '%s'"""\
         % cV


    print """\n>> Vectorised classifications maps for next step:\n"""

    # fix following to get rid of: [ ' , ']
    pprint.pprint(vectorised_maps, indent = 3)


# main function ##############################################################

def main():


    VectoriseReferences()

    VectoriseClassifications()


#  make it both a reusable module and a stand-alone utility
if __name__ == "__main__":
    options, flags = grass.parser()
    main()

