#!/usr/bin/env python
#-*- coding:utf-8 -*-

#%option
#% key: layer
#% type: integer
#% description: Layer name ...
#% answer: 1
#% required : yes
#%end

# import modules
import sys
import grass.script as grass




def main():
#def CoI_Percentages():
    """
    Step 4: calculate percentages of Class of Interest pixels
for each low resolution grid-cell
    """

    """
    Step 4.a: Preparative action: represent gridcells as points
(gridcell ~> extract centroids ~> convert to points)

    """
##############################################################################
### input: low resolution vector grid-cell

    # grids from which to pull cats
    group_of_lowres_vector_grids_raw = grass.read_command('g.mlist',\
                                        type = "vect",\
                                        pat = "pareto_grid___OVER___*___ON___*")
    group_of_lowres_vector_grids = group_of_lowres_vector_grids_raw.splitlines()

### intermediate: centroids of low resolution vector grid-cell

### output: points (=centroids) of low resolution grid-cell
##############################################################################

    print """\n > Representig gridcells as points; extracting gridcell centroids from:\n"""

    for lowres_vector_grid in group_of_lowres_vector_grids:

        lowres_vector_grid_centroids = lowres_vector_grid + "_centroids"
        lowres_vector_grid_points = lowres_vector_grid_centroids.replace("centroids", "points")

        # extract centroids from grid
        grass.run_command('v.extract',\
        flags = "-o",\
        quiet = True,\
        input = lowres_vector_grid,\
        output = lowres_vector_grid_centroids,\
        type = "centroid")
        print "   + %s"\
         % lowres_vector_grid


        # convert to points
        grass.run_command('v.type',\
        flags = "-o",\
        quiet = True,\
        input = lowres_vector_grid_centroids,\
        output = lowres_vector_grid_points,\
        type = "centroid,point")


        # remove centroids vector map
        grass.run_command('g.remove',\
        quiet = True,\
        vect = lowres_vector_grid_centroids)

    print """\n   * Gridcell centroids converted to points; temporal material removed"""


    """
    Step 4.b: calculate percentages
    """

    ### input: Class of Interest points (==centroids, ==pixels) of low resolution gridcell

    layer = options['layer']

    ### intermediate:
    ### output: percentage of Class of Interest pixels in each low-resolution gridcell


    print "\n > Computing percentage of coi pixels per low-resolution grid-cell \n"


    # grid_points for which to compute coi percentages
    group_of_lowres_vector_grid_points_raw = grass.read_command('g.mlist',\
                                        type = "vect",\
                                        pat = "pareto_grid___OVER___*_points")
    group_of_lowres_vector_grid_points = group_of_lowres_vector_grid_points_raw.splitlines()


    for lowres_vector_grid_points in group_of_lowres_vector_grid_points:

        # add "coi_percent" column
        coi_percent = "coi_percent"

        grass.run_command('v.db.addcol',\
        map = lowres_vector_grid_points,\
        column = 'coi_percent double')
        print "     * <%s> column added in  '%s'" % (coi_percent, lowres_vector_grid_points)

        # get important parameters
        vldb = grass.vector_layer_db(lowres_vector_grid_points, layer)

        table = vldb['table']
        keycol = vldb['key']
        database = vldb['database']
        driver = vldb['driver']


        # sql: populate "coi_percent" column = ( 100 / total points ) * CoI points
        sql_coi_percent = "UPDATE %s SET coi_percent = (coi_pix * 1. / total_pix *1. ) * 100"\
        % lowres_vector_grid_points


        # populate "coi_percent" column
        # for grass70 add: input = '-'
        grass.write_command('db.execute',\
        stdin = sql_coi_percent,\
        database = database,\
        driver = driver)
        print "     * Class of Interest percentage computed for %s (updated in <%s> column)\n"\
        % (lowres_vector_grid_points, coi_percent)


#  make it both a reusable module and a stand-alone utility
if __name__ == "__main__":
    options, flags = grass.parser()
    main()

