#!/usr/bin/env python
#-*- coding:utf-8 -*-

#%option
#% key: lowres
#% type: double
#% description: Pixel (low) resolution of map to be assessed
#% answer: 500
#% required: yes
#%end

    ##%option
    ##% key: lowres_vector_grid_name
    ##% type: string
    ##% gisprompt: new,vector,vector
    ##% key_desc: name
    ##% description: Name for low resolution vector grid map
    ##% required: no
    ##%end

#%option
#% key: thresholds_int
#% type: integer
#% key_desc: threshold values
#% description: Threshold values for Pareto optimal maps
#% multiple: yes
#% answer: 1, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100
#% required: yes
#%end


# import modules
import sys
import grass.script as grass


# Step ?: Populate Thresholds ################################################
def main():
#def PopulateThresholds():
    """
    Step 5: Populate threshold columns based on CoI pixels percentage
    """


##############################################################################
### input:



#    # get classification rasters from options
#    classification_rasters = options['classification_rasters'].split(',')

#     if not given then get from g.mlist
#     if classification_rasters == '':

### indent below if the above check will be used ###

    classification_rasters=''

    # hardcoded stuff --------------------------------------------------------
    ### classification_rasters=["test_eig_correlation","test_svd_raw_uncentered_unscaled"]
    # hardcoded stuff --------------------------------------------------------

     # if not given then get from g.mlist
    if classification_rasters == '':
        classification_rasters_raw = grass.read_command('g.mlist',\
                                                type = "rast",\
                                                pat = "pareto_classification_*")
        classification_rasters = classification_rasters_raw.splitlines()

    # get classification vectors
    #    classification_vectors[x] = "%s_vector" % classification_rasters[x]
    classification_vectors_raw = grass.read_command('g.mlist',\
                                                    type = "vect",\
                                                    pat = "pareto_classification_V___*")
    classification_vectors = classification_vectors_raw.splitlines()


    # thresholds integers
    thresholds_int = [1] + range(10,110,10)
    # thresholds_int = options['thresholds_int']

    # new list
    p_thresholds=[]

    # fill-it
    for x in range(0, len(thresholds_int)):
        p_thresholds.append("t%s" % thresholds_int[x])


                        # or: create new list with pareto thresholds
                    #    p_thresholds = thresholds_int
                    #    for x in range(0, len(thresholds_int),1):
                    #            p_thresholds[x] = "t%s" % thresholds_int[x]

    # create new list for...
    pt_n_classifications = p_thresholds + classification_rasters


    # grids from which to pull cats
    group_of_lowres_vector_grid_points_raw = grass.read_command('g.mlist',\
                                        type = "vect",\
                                        pat = "pareto_grid___OVER___*___ON___*points")
    group_of_lowres_vector_grid_points = group_of_lowres_vector_grid_points_raw.splitlines()

    lowres = options['lowres']


### output:
##############################################################################

    print "\n > Populating threshold ( %s ) and classification columns ( %s )..."\
    % (p_thresholds, classification_rasters)


    ## If "0" values exist in threshold columns (those are the txxx columns),
    ## then hadling NULL's won't have any effect because... 0 is not NULL :-)"

    # columns required for:
        # Pareto-optimal maps
        # Classification results (per classification method)

        # for each of the above add following (columns):
            # Oe/Ce control
            # Oe/Ce control_string
            # number of erroneous pixels


    for lowres_vector_grid_points in group_of_lowres_vector_grid_points:
        print "for %s" % lowres_vector_grid_points

        # add threshold columns

        for ptc in pt_n_classifications:

            # just checking -- REMOVE later!
            print "Feeding the <%s> column" % ptc

            grass.run_command('v.db.addcol',\
            map = lowres_vector_grid_points,\
            columns =\
            """%s integer,\
             %s_control integer,\
              %s_control_string varchar(10),\
               %s_erroneous_pix double precision"""\
            % (ptc, ptc, ptc, ptc))

        print "\n   + Oe/Ce columns added for '%s'"\
         % lowres_vector_grid_points


        # populate threshold columns

        for t in thresholds_int:


            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = "t%i" % t,\
            value = t,\
            where = "coi_percent >= %d"\
            % t)

        print "\n  * Threshold columns %s populated for %s"\
         % (p_thresholds, lowres_vector_grid_points)

## def PopulateClassifications():

        """
        Step ?: populate classification results update burned spots for example from final burned area map (= burned_areas__FINAL)
        """


        # careful region matching required
        # res=500 to align to grid-vector otherwise points near borders are omitted

        # ugly hack! #########################################################
        # make it a list
        glist = lowres_vector_grid_points.split("_")

        # count it length
        glist_len = len (glist)

        # del las element
        del glist[glist_len - 1]

        # use underscore as a separator
        sep = "_"

        # join elements
        lowres_vector_grid = sep.join(glist)
        ########################################################## ugly hack #


        # now zoom to lowres_vector_grid NOT to the Points vector!

        # Hamish' suggestion might be a better alternative ###################
        #
        # (in bash)
        # eval `g.region -g`
        # res=$nsres
        # g.region n=n+$res s=s-$res w=w-$res e=e+$res
        #
        # in other words:
        # a. get region of points
        # b. expand s, e, n, w values and feed g.region
        #
        ######################################################################

        grass.run_command('g.region',\
        flags = "a",\
        res = lowres,\
        vect = lowres_vector_grid)
        print """\n * Region matched to %s (> points-gridcell map == "v.what.vect" safe)"""\
         % lowres_vector_grid


        # loop over classification maps and update respective column(s)

        for cR in classification_rasters:

            # make sure column names are identical to GRASS raster map names
            grass.run_command('v.what.rast',\
            vect = lowres_vector_grid_points,\
            rast = cR,\
            column = cR)
            print "\n  * Classification columns of  '%s'  populated\n"\
             % lowres_vector_grid_points


# def main(): # in case of using two functions
#    PopulateThresholds()
#    PopulateClassifications()


#  make it both a reusable module and a stand-alone utility
if __name__ == "__main__":
    options, flags = grass.parser()
    main()

