## Pareto chart


# Start ######################################################################

# set working directory
# setwd("/geo/grassdb/peloponnese/R/R_files/csv/")
setwd("/geo/grassdb/spearfish60/R_files/csv/")

# Load data (.csv tables)
##############################################################################
# idea taken from: https://stat.ethz.ch/pipermail/r-help/2009-July/205682.html
#

    errors.list <- list.files ( pattern = "csv" )

    pareto.errors <- NULL

    for (i in errors.list) {

        new.csv <- read.csv (i)

            if (is.null (pareto.errors) ) {

                pareto.errors = new.csv

            } else {

                pareto.errors = rbind (pareto.errors , new.csv)

            }

    }

#
##############################################################################



# -%<---%<--%<---%<--%<---%<--%<---%<--%<---%<--%<---%<--%<---%<--%<---%<--%<- #
# code below for step-by-step use! Requires setting some "variables" below :-)
# -%<---%<--%<---%<--%<---%<--%<---%<--%<---%<--%<---%<--%<---%<--%<---%<--%<- #



# Preparative steps


	# library gtools requires to use mixedsort
	library(gtools)

	# required >>> define "data_for_pareto" , "reference_name" , position for smartlegend ( "leg_x" , "leg_y" )
	data_for_pareto <- pareto.errors # samples.all
	reference_name <- "coi samples from Landsat7 (SLC-off)"
	leg_x = "left"
	leg_y = "bottom"




# Create "names" for... all classifications, meaning both "pareto optimal classifications" and "(hard-) classifications":
    # leave out other columns: cat, row(n), col(n), *_pix (=total_pix, coi_pix, erroneous_pix), control, control_string
    ### WATCH OUT: classifications (=pareto + various user classification attempts)  !=  classification.methods (=user classification attempts)
    classifications <- colnames ( pareto.errors [- c ( 1:8 ,  grep ( "control" , colnames (pareto.errors) ), grep ( "pix" , colnames (pareto.errors) ) ) ] )


	# names for classification "methods" sorted logically :-)

	# Classification methods:

	#[1] "Eigenvectors on correlation of normalised values"
	#[2] "Eigenvectors on covariance of normalised values"
	#[3] "Eigenvectors on correlation of raw values"
	#[4] "Eigenvectors on covariance of raw values"
	#[5] "SVD on uncentered, unscaled normalised values with manual corrections"
	#[6] "SVD on centered, scaled normalised values"
	#[7] "SVD on uncentered, unscaled normalised values"
	#[8] "SVD on centered, unscaled normalised values"
	#[9] "SVD on centered, scaled raw values"
	#[10] "SVD on uncentered, unscaled raw values"
	#[11] "SVD on centered, unscaled raw values"



	# combine "methods" in a vector
    classification.methods <- colnames ( pareto.errors [- c ( 1:8 ,  grep ( "^t.?.?.?.$" , colnames ( pareto.errors ) ) , grep ( "control" , colnames (pareto.errors) ), grep ( "pix" , colnames (pareto.errors) ) ) ] )

	# for use with text() or smartlegend() later
    # how/ when to ask for user input? # for now: classification.methods == classification.methods.legend
    classification.methods.legend = classification.methods



	# names for thresholds
	thresholds.names <- c ( "1%", "10%" , "20%" , "30%" , "40%" , "50%" , "60%" , "70%" , "80%" , "90%" , "100%")


# Beginning the real process...


	# load "the"object with actual classification values  (coi / NA)
	classifications <- data_for_pareto [ , classifications ] # use names(classifications) to use classification names in "for" loops


	# get total reference pixels
	reference.total_pixels <- sum ( data_for_pareto [ , "total_pix" ] , na.rm = TRUE )


	# get reference coi pixels
	reference.coi_pixels <- sum ( data_for_pareto [ , "coi_pix" ] , na.rm = TRUE  )



	# get coi pixels for each classification (incl. the pareto-optimal maps)
	for ( C in names (classifications) ) {

		assign (

			paste ( C , ".classified_coi_pixels" , sep = "" ) ,

				sum (

					subset ( data_for_pareto , !is.na ( data_for_pareto [[C]] ) , select = "total_pix") [,1] ,

				na.rm = TRUE )

		)

	}

	rm(C)



	# get accurate pixels for each classification (incl. the pareto-optimal maps)
	for ( C in names (classifications) ) {

		assign (

			paste ( C , ".accurate_pixels" , sep = "" ) , (

				reference.total_pixels - sum (

					 data_for_pareto [[ paste ( C , "_erroneous_pix" , sep = "" ) ]] ,

				na.rm = TRUE )

			)

		)

	}

	rm(C)




# compute Total accuracies

    #
	for ( C in names ( classifications ) ) {

		assign(

			paste ( C , ".accuracy" , sep = "" ) ,

				get (

					paste ( C , ".accurate_pixels" , sep = "" )

				)

			/ reference.total_pixels

		)

	}

accuracies <- list ( objects ( pattern = ".accuracy" ) )




	# group accuracies
	total.accuracies <- NULL

	for ( x in 1 : length ( unlist ( accuracies ) ) ) {

		total.accuracies [x] <- get ( mixedsort ( unlist ( accuracies ) ) [x] )

		names ( total.accuracies ) [x] <- mixedsort ( unlist ( accuracies ) ) [x]

}




	# extract accuracies.pareto
	accuracies.pareto <- as.data.frame ( 100 * total.accuracies [ c ( grep ( unlist ( accuracies ) , pattern = "^t.*.$" ) ) ] )

	names ( accuracies.pareto ) <- "Overall Accuracy"

	rownames ( accuracies.pareto ) <- thresholds.names



	# extract accuracies.classifications

	accuracies.classifications <- as.data.frame ( 100 * total.accuracies [ c ( grep ( unlist ( accuracies ) , pattern = "^pareto.*.$" ) ) ] )

	names ( accuracies.classifications ) <- "Overall Accuracy"

	rownames ( accuracies.classifications ) <- classification.methods




# compute Omission / Commission errors for all maps


  # count omissions
  for ( C in names ( classifications ) ) {

	assign (

		paste ( C , ".omissions" , sep = "" ) ,

			sum (

				subset ( data_for_pareto , data_for_pareto [[ paste (C , "_control_string" , sep = "" ) ]] == "omission" , select = paste ( C , "_erroneous_pix" , sep = "" ) ) [,1] ,

			na.rm = TRUE )

	)

# print ( get ( paste ( C , ".omissions" , sep = "" ) ) )

}

omissions <- list ( objects ( pattern = ".omissions" ) )




  # count commissions
  for ( C in names ( classifications ) ) {

	assign (

		paste ( C , ".commissions" , sep = "" ) ,

			sum (

				subset ( data_for_pareto , data_for_pareto [[ paste (C , "_control_string" , sep = "" ) ]] == "commission" , select = paste ( C , "_erroneous_pix" , sep = "" ) ) [,1] ,

			na.rm = TRUE )

	 )

# print ( get ( paste ( C , ".commissions" , sep = "" ) ) )

  }

commissions <- list ( objects ( pattern = ".commissions" ) )

	# remove CCS
	rm(C)




# compute Omission errors (Oe)

	# Oe <- omissions / number of reference coi pixels
	for ( C in names ( classifications ) ) {

	assign (

		paste ( "Oe." , C , sep = "") , (

			get (

				paste (	C , ".omissions" , sep = "" )

			)

		/ reference.coi_pixels )

	)

# print ( get ( paste ( "Oe." , C , sep = "") ) )

	}

rm(C)



# compute Commission errors (Oe)

	# Ce <- commissions / number of classified coi pixels
	for ( C in names (classifications) ) {

	assign (

		paste ( "Ce." , C , sep = "") , (

			get (

				paste ( C , ".commissions" , sep = "" )

			 )

		/

			get (

				paste ( C , ".classified_coi_pixels" , sep = "" )

			)

		 )

	)

# print ( get ( paste ( "Ce." , C , sep = "") ) )

	}

rm(C)



# group Oe/Ce errors in a matrix


	# group ommission errors for pareto-optimal maps
	omission.errors.pareto <- NULL

	for ( x in 1:length ( objects ( pattern="^Ce.t.*.$" ) ) ) {

		omission.errors.pareto[x] <- get ( mixedsort ( objects ( pattern="^Oe.t.*.$" ) ) [x])

		names(omission.errors.pareto)[x] <- mixedsort ( objects ( pattern="^Oe.t.*.$" ) ) [x]

	}


	# group ommission errors for classification maps
	omission.errors.classifications <- NULL

	for ( x in 1:length ( objects ( pattern = "^Oe.pareto.*.$" ) ) ) {

		omission.errors.classifications[x] <- get ( mixedsort ( objects ( pattern = "^Oe.pareto.*.$" ) ) [x])

		names(omission.errors.classifications)[x] <- mixedsort ( objects ( pattern = "^Oe.pareto.*.$" ) ) [x]

	}


	# group commission errors for pareto-optimal maps
	commission.errors.pareto <- NULL

	for ( x in 1:length ( objects ( pattern = "^Ce.t.*.$" ) ) ) {

		commission.errors.pareto[x] <- get ( mixedsort ( objects ( pattern="^Ce.t.*.$" ) ) [x])

		names(commission.errors.pareto)[x] <- mixedsort ( objects ( pattern="^Ce.t.*.$" ) ) [x]

	}


	# group commission errors for classifications maps
	commission.errors.classifications <- NULL

	for ( x in 1:length ( objects ( pattern = "^Ce.pareto.*.$" ) ) ) {

		commission.errors.classifications[x] <- get ( mixedsort ( objects ( pattern="^Ce.pareto.*.$") ) [x])

		names(commission.errors.classifications)[x] <- mixedsort ( objects ( pattern="^Ce.pareto.*.$" ) ) [x]

	}



	# cbind Oe/Ce for pareto-optimal maps as data.frame
	Oe.Ce.Pareto <- cbind.data.frame ( omission.errors.pareto , commission.errors.pareto)

	names(Oe.Ce.Pareto) <- c ( "Omission errors" , "Commission errors" )

	rownames ( Oe.Ce.Pareto ) <- thresholds.names




# compute Producer's and User's accuracy

	# Producer's and User's accuracy for Pareto-Optimal maps ( Pa = 1 - Oe , Ua = 1 - Ce )

	# Does this make any sense?
	Pa.Ua.Pareto <- ( 1 - Oe.Ce.Pareto ) * 100

	names ( Pa.Ua.Pareto ) <- c ( "Producer's accuracy" , "User's accuracy" )

	rownames ( Pa.Ua.Pareto ) <- thresholds.names



	# cbind Oe/Ce for Classification maps as data.frame
	Oe.Ce.Classifications <- cbind.data.frame ( omission.errors.classifications , commission.errors.classifications)

	names(Oe.Ce.Classifications) <- c ( "Omission errors" , "Commission errors" )

	rownames ( Oe.Ce.Classifications ) <- classification.methods



	# Producer's and User's accuracy for Classification maps ( Pa = 1 - Oe , Ua = 1 - Ce )
	Pa.Ua.Classifications <- ( 1 - Oe.Ce.Classifications ) * 100

	names ( Pa.Ua.Classifications ) <- c ( "Producer's accuracy" , "User's accuracy" )

	rownames ( Pa.Ua.Classifications ) <- classification.methods




# Bind together final statistics for pareto-optimal maps and classifications

    #
	statistics.pareto <- cbind.data.frame ( Pa.Ua.Pareto , accuracies.pareto )

	statistics.classifications <- cbind.data.frame ( Pa.Ua.Classifications , accuracies.classifications )

	statistics.final <- list ( statistics.pareto , statistics.classifications )

